package javax.swing.text.rtf;

import java.io.Reader;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

abstract class AbstractFilter extends OutputStream
{
    protected char[] translationTable;
    protected boolean[] specialsTable;
    static final char[] latin1TranslationTable;
    static final boolean[] noSpecialsTable;
    static final boolean[] allSpecialsTable;
    
    public void readFromStream(InputStream inputStream) throws IOException {
        byte[] b = new byte[16384];
        while (true) {
            int read = inputStream.read(b);
            if (read < 0) {
                break;
            }
            this.write(b, 0, read);
        }
    }
    
    public void readFromReader(Reader reader) throws IOException {
        char[] cbuf = new char[2048];
        while (true) {
            int read = reader.read(cbuf);
            if (read < 0) {
                break;
            }
            for (int i = 0; i < read; ++i) {
                this.write(cbuf[i]);
            }
        }
    }
    
    public AbstractFilter() {
        this.translationTable = AbstractFilter.latin1TranslationTable;
        this.specialsTable = AbstractFilter.noSpecialsTable;
    }
    
    @Override
    public void write(int n) throws IOException {
        if (n < 0) {
            n += 256;
        }
        if (this.specialsTable[n]) {
            this.writeSpecial(n);
        }
        else {
            int n2 = this.translationTable[n];
            if (n2 != '\0') {
                this.write((char)n2);
            }
        }
    }
    
    @Override
    public void write(byte[] array, int n, int i) throws IOException {
        Object o = null;
        while (i > 0) {
            int n2 = (short)array[n];
            if (n2 < 0) {
                n2 = (short)(n2 + 256);
            }
            if (this.specialsTable[n2]) {
                if (o != null) {
                    this.write(o.toString());
                    o = null;
                }
                this.writeSpecial(n2);
            }
            else {
                int c = this.translationTable[n2];
                if (c != '\0') {
                    if (o == null) {
                        o = new StringBuilder();
                    }
                    o.append((char)c);
                }
            }
            --i;
            ++n;
        }
        if (o != null) {
            this.write(o.toString());
        }
    }
    
    public void write(String s) throws IOException {
        for (int length = s.length(), i = 0; i < length; ++i) {
            this.write(s.charAt(i));
        }
    }
    
    protected abstract void write(char p0) throws IOException;
    
    protected abstract void writeSpecial(int p0) throws IOException;
    
    static {
        noSpecialsTable = new boolean[256];
        for (int i = 0; i < 256; ++i) {
            AbstractFilter.noSpecialsTable[i] = false;
        }
        allSpecialsTable = new boolean[256];
        for (int j = 0; j < 256; ++j) {
            AbstractFilter.allSpecialsTable[j] = true;
        }
        latin1TranslationTable = new char[256];
        for (int k = 0; k < 256; ++k) {
            AbstractFilter.latin1TranslationTable[k] = (char)k;
        }
    }
}
